import React, { useState } from 'react';
import ReactDOM from 'react-dom';

import { scaleLinear, extent, scaleOrdinal } from 'd3';

import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { DropDownContainer } from './DropDownContainer';
import { ColorLegend } from './ColorLegend';

import 'react-dropdown/style.css';

// dimensions & margins
const width = 960;
const height = 500;
const margin = { top: 20, right: 150, bottom: 200, left: 110 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

// parameters
const axisOffset = 32;
const circleR = 10;
const legendOffset = 25;
const markOpacity = 0.8;

// variables
const vars = [
  { value: 'wt', label: 'Weight (lb)', type: 'num' },
  { value: 'mpg', label: 'Miles per gallon', type: 'num' },
  { value: 'disp', label: 'Engine displacement (cu in)', type: 'num' },
  { value: 'cyl', label: 'Number of cylinders', type: 'cat' },
  { value: 'am', label: 'Transmission', type: 'cat' },
];

const getLabel = (val) => vars.filter((v) => v.value === val)[0].label;

function App() {
  // data & state
  const data = useData();

  const initXAttr = 'wt';
  const initYAttr = 'mpg';
  const initCAttr = 'cyl';
  const [xAttr, setXAttr] = useState(initXAttr);
  const [yAttr, setYAttr] = useState(initYAttr);
  const [cAttr, setCAttr] = useState(initCAttr);
  const [hoveredVal, setHoveredVal] = useState(null);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  // scales
  const xValue = (d) => d[xAttr];
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yValue = (d) => d[yAttr];
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice(5);

  const cValue = (d) => d[cAttr];
  const cScale = scaleOrdinal()
    .domain(data.map(cValue))
    .range(['steelblue', 'tomato', 'goldenrod']);

  return (
    <div className="App">
      <h2>
        Exploratory Scatterplot of <code>mtcars</code>
      </h2>

      <DropDownContainer
        xAttr={xAttr}
        setXAttr={setXAttr}
        yAttr={yAttr}
        setYAttr={setYAttr}
        cAttr={cAttr}
        setCAttr={setCAttr}
        getLabel={getLabel}
        vars={vars}
      />

      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <ColorLegend
            cScale={cScale}
            innerWidth={innerWidth}
            axisOffset={axisOffset}
            legendOffset={legendOffset}
            circleR={circleR}
            markOpacity={markOpacity}
            hoveredVal={hoveredVal}
            setHoveredVal={setHoveredVal}
          />

          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            axisOffset={axisOffset}
          />

          <text
            className="label"
            x={innerWidth}
            y={innerHeight + 2 * axisOffset}
          >
            {getLabel(xAttr)}
          </text>

          <text
            className="label"
            x={-axisOffset}
            transform={`translate(
                ${-2 * axisOffset},
                ${-axisOffset})
              rotate(-90)`}
          >
            {getLabel(yAttr)}
          </text>

          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            axisOffset={axisOffset}
          />

          <Marks
            data={data}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            cValue={cValue}
            cScale={cScale}
            circleR={circleR}
            markOpacity={markOpacity}
            hoveredVal={hoveredVal}
          />
        </g>
      </svg>
    </div>
  );
}

export default App;

ReactDOM.render(<App />, document.getElementById('root'));
